# vim:sw=4 sta showmatch

# A poor imitation of SGMLSpm / SGMLSpl using PerlSAX.
# [steve@ggi-project.org]

use strict;

package XML::SGMLSpl::Element;

sub new {
    my ($class, $gi, $attrs, $parent) = @_;
    my $self = [ $gi, $parent, $attrs, {} ];
    return bless($self, $class);
}

sub name 	{ $_[0]->[0] }
sub parent 	{ $_[0]->[1] }
sub attributes 	{ $_[0]->[2] }
sub attribute 	{ $_[0]->[2]->{$_[1]} }
sub ext 	{ $_[0]->[3] }

sub within {
    my ($self,$name) = @_;
    for ($self = $self->parent; $self; $self = $self->parent) {
	return $self if ($self->name eq $name);
    }

    return undef;
}

sub in {
    my ($self,$name) = @_;
    if ($self->parent && $self->parent->name eq $name) {
	return $self->parent;
    } else {
	return undef;
    }
}
				
package XML::SGMLSpl;

# uses PerlSAX interface

sub new {
    my($class) = @_;
    my $self = {};
    return bless($self, $class);
}
			 
sub start_document {
    my ($self) = @_;
    &{($self->{'start_document'}->{''} || sub{})}($self);
}

sub end_document {
    my ($self) = @_;
    &{($self->{'end_document'}->{''} || sub{})}($self);
}


sub start_element {
    my ($self, $arg) = @_;
    my $elem = new XML::SGMLSpl::Element(
	$arg->{'Name'}, 
	$arg->{'Attributes'},
	$self->{'current_element'});
    $self->{'current_element'} = $elem;

    &{($self->{'start_element'}->{$arg->{'Name'}} || 
       $self->{'start_element'}->{''} || sub{})}($elem, $self);
}

sub end_element {
    my ($self, $arg) = @_;
    my $elem = $self->{'current_element'};

    &{($self->{'end_element'}->{$arg->{'Name'}} ||
       $self->{'end_element'}->{''} || sub{})}($elem, $self);

    # unlink after
    $self->{'current_element'} = $elem->parent;
}

sub characters {
    my ($self, $arg) = @_;

    &{($self->{'characters'} || sub{})}
	($arg->{'Data'}, $self->{'current_element'}, $self);
}

sub processing_instruction {
    my ($self, $arg) = @_;
    &{($self->{'processing_instruction'} || sub{})}
	($arg->{'Target'}, $arg->{'Data'}, $self);
}

sub ignorable_whitespace {
    my ($self, $arg) = @_;

    &{($self->{'ignorable_whitespace'} || sub{})}
	($arg->{'Data'}, $self->{'current_element'}, $self);
}


# stolen from
########################################################################
# sgmlspl: a simple SGML postprocesser for the SGMLS and NSGMLS
#          parsers (requires SGMLS.pm library).
#
# Copyright (c) 1995 by David Megginson <dmeggins@aix1.uottawa.ca>

# The generic events are as follow:
#
#   'start'                 The beginning of the document.
#   'end'                   The end of the document.
#   'start_element'         The beginning of an element.
#   'end_element'           The end of an element.
#   'cdata'                 Regular character data.
#   'sdata'                 Special system-specific data.
#   're'                    A record-end.
#   'pi'                    A processing instruction.
#   'entity'                An external-entity reference.
#   'start_subdoc'          The beginning of a subdocument entity.
#   'end_subdoc'            The end of a subdocument entity.
#   'conforming'            The document is conforming.
#
# In addition to these generic events, it is possible to handlers
# for certain specific, named events, as follow:
#
#   '<GI>'                  The beginning of element GI.
#   '</GI>'                 The end of element GI.
#   '[SDATA]'               The system-specific data SDATA.
#   '&ENAME;'               A reference to the external entity ENAME.
#   '{ENAME}'               The beginning of the subdocument-entity ENAME.
#   '{/ENAME}'              The end of the subdocument-entity ENAME.
#
sub sethandler {

    my $handler = pop(@_);
    my ($self, $spec) = @_;
    
    SWITCH: {
				# start-document handler
      $spec eq 'start' && do {

	  $self->{'start_document'} = $handler;
	  last SWITCH;
      };
				# end-document handler
      $spec eq 'end' && do {
	  $self->{'$end_document'} = $handler;
	  last SWITCH;
      };
				# start-element handler
      $spec =~ /^<([^\/].*|)>$/ && do {
	  $self->{'start_element'}->{$1} = $handler;
	  last SWITCH;
      };
				# generic start-element handler
      $spec eq 'start_element' && do {
	  $self->{'start_element'}->{''} = $handler;
	  last SWITCH;
      };
				# end-element handler
      $spec =~ /^<\/(.*)>$/ && do {
	  $self->{'end_element'}->{$1} = $handler;
	  last SWITCH;
      };
				# generic end-element handler
      $spec =~ 'end_element' && do {
	  $self->{'$end_element'}->{''} = $handler;
	  last SWITCH;
      };
				# cdata handler
      $spec eq 'characters' && do {
	  $self->{'characters'} = $handler;
	  last SWITCH;
      };

      $spec eq 'ignorable_whitespace' && do {
	  $self->{'ignorable_whitespace'} = $handler;
	  last SWITCH;
      };
				# processing-instruction handler
      $spec eq 'processing_instruction' && do {
	  $self->{'processing_instruction'} = $handler;
	  last SWITCH;
      };
	
      die "Bad XML::Process handler pattern: $spec\n";
  }
}

1;

